/* 
 *           DO WHAT THE FUCK YOU WANT TO PUBLIC LICENSE
 *                    Version 2, December 2004
 *  
 * Copyright (C) 2004 Sam Hocevar <sam@hocevar.net>
 * 
 * Everyone is permitted to copy and distribute verbatim or modified
 * copies of this license document, and changing it is allowed as long
 * as the name is changed.
 *  
 *            DO WHAT THE FUCK YOU WANT TO PUBLIC LICENSE
 *   TERMS AND CONDITIONS FOR COPYING, DISTRIBUTION AND MODIFICATION
 *  
 *  0. You just DO WHAT THE FUCK YOU WANT TO.
 */
/* 
 * Nazo No Masquerade image encoder
 * author : MooZ
 */
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <errno.h>

#include "image.h"
#include "pcx.h"

/**
 * Encode an 16 colors pcx file for Nazo no Mascarade.
 **/
 int encode_img(Image* src, FILE* out)
 {
    int count, y, x, offset;
    uint8_t byte;
    size_t nwritten;

    for(y=0; y<src->height; y++)
    {
        for(x=0, offset=y*src->width; x<src->width; x+=count, offset+=count)
        {
            byte = src->data[offset];
            if(byte >= 16)
            {
                fprintf(stderr, "Invalid input. The image must contain at most 16 colors!\n");
                return 0;
            }
        
            for(count=1; (((x+count)<src->width) && (count < 15)) && (byte == src->data[offset+count]); count++)
            {}

            byte &= 0x0f;
            byte |= (count & 0x0f) << 4;

            nwritten = fwrite(&byte, 1, 1, out);
            if(nwritten != 1)
            {
                fprintf(stderr, "Failed to write byte : %s\n", strerror(errno));
                return 0;
            }
        }
    }
    
    return 1;
 }
 
 int main(int argc, char** argv)
 {
    Image img;
    FILE *out;
    char *out_filename;
    size_t out_len;
    int ret, offset;
    int err = 1;
  
    if(argc != 3)
    {
        fprintf(stderr, "Usage : %s pcx out\n", argv[0]);
        goto err_0;
    }

    out_len = strlen(argv[2]);
    out_filename = (char*)malloc(out_len + 5);
    if(out_filename == NULL)
    {
        fprintf(stderr, "Failed to allocate memory: %s\n", strerror(errno));
        goto err_1;
    }

    ret = pcx_load(argv[1], &img);
    if(!ret)
    {
        goto err_2;
    }
    
    if((img.bytes_per_pixel != 1) && (img.color_count != 256))
    {
        fprintf(stderr, "Invalid pcx file %s.\n", argv[1]);
        goto err_2;
    }

    strcpy(out_filename, argv[2]);
    strcpy(out_filename+out_len, ".dat");
    
    out = fopen(out_filename, "wb");
    if(out == NULL)
    {
        fprintf(stderr, "Failed to open %s : %s\n", out_filename, strerror(errno));
        goto err_2;
    }

    /* Output data */
    ret = encode_img(&img, out);
    if(!ret)
    {
        goto err_3;
    }
    
    fclose(out);

    strcpy(out_filename+out_len, ".pal");
    
    out = fopen(out_filename, "wb");
    if(out == NULL)
    {
        fprintf(stderr, "Failed to open %s : %s\n", out_filename, strerror(errno));
        goto err_2;
    }

    /* Output palette */
    for(offset=0; offset<16*3; offset+=3)
    {
        fprintf(out, "%02x, %02x, %02x\n", 
                img.palette[offset  ],
                img.palette[offset+1],
                img.palette[offset+2]);
    }
       
    err = 0;
err_3:     
    fclose(out);    
err_2:
    destroy_image(&img);
err_1:
    free(out_filename);
err_0:
    return err;
}